/*
 * Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.oracle.truffle.r.library.fastrGrid.grDevices;

import java.io.IOException;

import com.oracle.truffle.api.CompilerDirectives.TruffleBoundary;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.r.library.fastrGrid.GridContext;
import com.oracle.truffle.r.library.fastrGrid.device.GridDevice;
import com.oracle.truffle.r.library.fastrGrid.device.ImageSaver;
import com.oracle.truffle.r.nodes.builtin.RExternalBuiltinNode;
import com.oracle.truffle.r.runtime.RError.Message;
import com.oracle.truffle.r.runtime.data.RNull;

public abstract class SavePlot extends RExternalBuiltinNode.Arg3 {
    static {
        Casts casts = new Casts(SavePlot.class);
        casts.arg(0).asStringVector().findFirst();
        casts.arg(1).asStringVector().findFirst();
    }

    public static SavePlot create() {
        return SavePlotNodeGen.create();
    }

    @Specialization
    @TruffleBoundary
    Object savePlot(String filename, String type) {
        GridDevice device = GridContext.getContext().getCurrentDevice();
        if (!(device instanceof ImageSaver)) {
            throw error(Message.GENERIC, "Current device does not support plot saving. " +
                            "Note that FastR savePlot function ignores the device argument and always uses the current device.");
        }
        try {
            ((ImageSaver) device).save(filename, type);
        } catch (IOException e) {
            throw error(Message.GENERIC, "I/O error occured when saving the image.");
        }
        return RNull.instance;
    }
}
